% Autor: Wiebke Petersen
% Datum: 11.05.2017
% loaded by sudoku_test_suite.pl

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In order to get a solution in a reasonable time, the following
% predicate has to be improved!
% Alternatively one should reduce the number of boards in sudoku_boards.pl.

% the working horse: games are played and solved
% try out alternative definitions of play/2

play(M,M):- length(M,81), !.    % matrix completed - problem solved
play(M,Solution):-
        choose_some_value(RCV,M), % make a temporary decision
        consistent(RCV,M,NewM), % if consistent, NewM appears
        play(NewM,Solution).    % matrix enhanced with new element

choose_some_value((R,C,V),M):-
        numbers(N),
        member(R,N),
        member(C,N),
        \+ member((R,C,_),M),!,    % make sure that the field is empty
        member(V,N).

consistent((R,C,V),M,[(R,C,V)|M]):-
        row_consistent((R,V),M),
        col_consistent((C,V),M),
        block_consistent((R,C,V),M).

row_consistent((R,V),M):-
        \+ member((R,_,V),M).

col_consistent((C,V),M):-
        \+ member((_,C,V),M).

block_consistent((R,C,V),M):-
        block_values((R,C),M,Values),
        \+ member(V,Values).

block_values((R,C),M,Values):-
        interval(R,RInt),
        interval(C,CInt),
        findall(V,(member(R1,RInt),member(C1,CInt),member((R1,C1,V),M)),Values).

interval(I,Int):-
        numbers(N),
        findall(Z,(member(Z,N),(I-1)//3 =:= (Z-1)//3),Int).

numbers([1,2,3,4,5,6,7,8,9]).


%%%%%%%
% print board

print_board(B):- nl, print_board(B,1,1).

print_board(_,10,_):- nl,nl,!. % board is done
print_board(B,R,10):- !,    % row is done
        nl,
        R1 is R+1,
        print_board(B,R1,1). % repeat with next row

print_board(B,R,C):-
        member((R,C,V),B),!, % field is filled
        write(V),tab(1),     % print value
        C1 is C+1,
        print_board(B,R,C1). % go to next field

print_board(B,R,C):-
        write("_"),tab(1),   % print empty field
        C1 is C+1,
        print_board(B,R,C1). % go to next field
